@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/cmwind/CMWcmmnds.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/cmwind/CMWcmmnds.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -272,7 +272,7 @@ cmwButtonUp(w, p, button)
 
 	/* Mark all windows containing this color as modified */
 	(void) WindSearch(CMWclientID, (ClientData) NULL, (Rect *) NULL,
-		cmwRedisplayFunc, (ClientData) crec->cmw_color);
+		cmwRedisplayFunc, (ClientData)(pointertype) crec->cmw_color);
     }
 }
 
@@ -585,7 +585,7 @@ cbUpdate(w, code, x, replace)
 
     /* Mark all windows containing this color as modified */
     (void) WindSearch(CMWclientID, (ClientData) NULL, (Rect *) NULL,
-		cmwRedisplayFunc, (ClientData) cr->cmw_color);
+		cmwRedisplayFunc, (ClientData)(pointertype) cr->cmw_color);
 
     /* Here. . . ought to mark the layout windows as modified? */
 }
