@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBcellsrch.c,v 1.6 2010/08/15 14:35:47 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBcellsrch.c,v 1.7 2010/09/15 21:53:22 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -204,7 +204,8 @@ dbCellTileSrFunc(scx, fp)
 	    context.tc_plane = pNum;
 	    if (fp->tf_dinfo & TT_DIAGONAL)
 	    {
-		TileType dinfo = DBTransformDiagonal(fp->tf_dinfo, &scx->scx_trans);
+		// TileType dinfo = DBTransformDiagonal(fp->tf_dinfo, &scx->scx_trans);
+		TileType dinfo = DBInvTransformDiagonal(fp->tf_dinfo, &scx->scx_trans);
 		if (DBSrPaintNMArea((Tile *) NULL, def->cd_planes[pNum],
 			dinfo, &scx->scx_area, fp->tf_mask,
 			fp->tf_func, (ClientData) &context))
@@ -1665,6 +1666,10 @@ dbScaleCell(cellDef, scalen, scaled)
 	DBScalePoint(&bbox->r_ll, scalen, scaled);
 	DBScalePoint(&bbox->r_ur, scalen, scaled);
 
+	bbox = &use->cu_extended;
+	DBScalePoint(&bbox->r_ll, scalen, scaled);
+	DBScalePoint(&bbox->r_ur, scalen, scaled);
+
 	DBScaleValue(&use->cu_transform.t_c, scalen, scaled);
 	DBScaleValue(&use->cu_transform.t_f, scalen, scaled);
 
@@ -1744,6 +1749,8 @@ donecell:
 
     DBScalePoint(&cellDef->cd_bbox.r_ll, scalen, scaled);
     DBScalePoint(&cellDef->cd_bbox.r_ur, scalen, scaled);
+    DBScalePoint(&cellDef->cd_extended.r_ll, scalen, scaled);
+    DBScalePoint(&cellDef->cd_extended.r_ur, scalen, scaled);
 
     return 0;
 }
