@@ -977,7 +977,7 @@ dbReadOpen(cellDef, name, setFileName)
 			 * cell was loaded.
 			 */
 {
-    FILE *f;
+    FILE *f = NULL;
     char *filename, *realname;
     bool is_locked;
 
@@ -1255,7 +1255,7 @@ badTransform:
     subCellDef = DBCellLookDef(cellname);
     if (subCellDef == (CellDef *) NULL)
     {
-	subCellDef = DBCellNewDef(cellname, (char *) NULL);
+	subCellDef = DBCellNewDef(cellname, (char *)NULL);
 	subCellDef->cd_timestamp = childStamp;
 
 	/* Make sure rectangle is non-degenerate */
@@ -2102,6 +2102,7 @@ DBCellWriteFile(cellDef, f)
     FILE *f;		/* The FILE to write to */
 {
     int dbWritePaintFunc(), dbWriteCellFunc(), dbWritePropFunc();
+    int dbClearCellFunc();
     Label *lab;
     struct writeArg arg;
     int pNum;
@@ -2187,6 +2188,9 @@ DBCellWriteFile(cellDef, f)
     if (DBCellEnum(cellDef, dbWriteCellFunc, (ClientData) &arg))
 	goto ioerror;
 
+    /* Clear flags set in dbWriteCellFunc */
+    DBCellEnum(cellDef, dbClearCellFunc, (ClientData)NULL);
+
     /* Now labels */
     if (cellDef->cd_labels)
     {
@@ -2419,12 +2423,14 @@ DBCellWrite(cellDef, fileName)
      */
     if (fileName)
     {
-	realname = (char *) mallocMagic((unsigned) (strlen(fileName)+strlen(DBSuffix)+1));
+	realname = (char *) mallocMagic(strlen(fileName) + strlen(DBSuffix) + 1);
 	(void) sprintf(realname, "%s%s", fileName, DBSuffix);
+
 	/* Bug fix: 7/17/99, Michael D. Godfrey:  Forces		*/
 	/* cd_name and cd_file to ALWAYS be the same, otherwise ugly	*/
 	/* surprises can occur after saving a file as a different	*/
 	/* filename.							*/
+
 	cellDef->cd_file = StrDup(&cellDef->cd_file, realname);
     }
     else if (cellDef->cd_file)
@@ -2463,6 +2469,7 @@ DBCellWrite(cellDef, fileName)
 #else
 	TxPrintf("File %s is read_only and cannot be written\n", realname);
 #endif
+	freeMagic(realname);
 	return(FALSE);
     }
 
@@ -2732,6 +2739,31 @@ dbWritePaintFunc(tile, cdarg)
     return 0;
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * dbClearCellFunc --
+ *
+ * Filter function that clears flags set by dbWriteCellFunc.
+ *
+ * Results:
+ *	Always returns 0
+ *
+ * Side effects:
+ *	Cell use flags changed.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+dbClearCellFunc(cellUse, cdarg)
+    CellUse *cellUse;	/* Cell use */
+    ClientData cdarg;	/* Not used */
+{
+    cellUse->cu_def->cd_flags &= ~CDVISITED;
+    return 0;
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -2757,15 +2789,36 @@ dbWriteCellFunc(cellUse, cdarg)
     struct writeArg *arg = (struct writeArg *) cdarg;
     Transform *t;
     Rect *b;
-    char     cstring[256];
+    char     cstring[256], *pathend;
 
     t = &(cellUse->cu_transform);
     b = &(cellUse->cu_def->cd_bbox);
-    sprintf(cstring, "use %s %c%s\n", cellUse->cu_def->cd_name,
+
+    if (cellUse->cu_def->cd_file == NULL)
+	pathend = NULL;
+    else
+    {
+	pathend = strrchr(cellUse->cu_def->cd_file, '/');
+	if (pathend != NULL) *pathend = '\0';
+    }
+
+    if ((cellUse->cu_def->cd_flags & CDVISITED) || (pathend == NULL))
+    {
+	sprintf(cstring, "use %s %c%s\n", cellUse->cu_def->cd_name,
 		(cellUse->cu_flags & CU_LOCKED) ? CULOCKCHAR : ' ',
 		cellUse->cu_id);
+    }
+    else
+    {
+	sprintf(cstring, "use %s %c%s %s\n", cellUse->cu_def->cd_name,
+		(cellUse->cu_flags & CU_LOCKED) ? CULOCKCHAR : ' ',
+		cellUse->cu_id, cellUse->cu_def->cd_file);
+    }
     FPRINTR(arg->wa_file, cstring);
 
+    cellUse->cu_def->cd_flags |= CDVISITED;
+    if (pathend != NULL) *pathend = '/';
+
     if ((cellUse->cu_xlo != cellUse->cu_xhi)
 	    || (cellUse->cu_ylo != cellUse->cu_yhi))
     {
