@@ -263,7 +263,7 @@ spcdevHierVisit(hc, dev, scale)
     DevParam *plist, *pptr;
     DevTerm *gate, *source, *drain;
     EFNode  *subnode, *snode, *dnode, *subnodeFlat = NULL;
-    int l, w, i;
+    int l, w, i, parmval;
     Rect r;
     bool subAP= FALSE, hierS, hierD, extHierSDAttr() ;
     float sdM; 
@@ -347,6 +347,7 @@ spcdevHierVisit(hc, dev, scale)
 	    break;
 	case DEV_SUBCKT:
 	case DEV_RSUBCKT:
+	case DEV_MSUBCKT:
 	    devchar = 'X';
 	    break;
     }
@@ -375,6 +376,7 @@ spcdevHierVisit(hc, dev, scale)
 		break;
 	    case DEV_SUBCKT:
 	    case DEV_RSUBCKT:
+	    case DEV_MSUBCKT:
 		fprintf(esSpiceF, "%d", esSbckNum++);
 		break;
 	    default:
@@ -411,6 +413,17 @@ spcdevHierVisit(hc, dev, scale)
 	    fprintf(esSpiceF, " %s", EFDevTypes[dev->dev_type]);
 	    break;
 
+	case DEV_MSUBCKT:
+	    /* msubcircuit is "Xnnn source gate [drain [sub]]]"		*/
+	    /* to more conveniently handle situations where MOSFETs	*/
+	    /* are modeled by subcircuits with the same pin ordering.	*/
+
+	    spcdevOutNode(hc->hc_hierName,
+			source->dterm_node->efnode_name->efnn_hier,
+			"subckt", esSpiceF);
+
+	    /* Drop through to below (no break statement) */
+
 	case DEV_SUBCKT:
 
 	    /* Subcircuit is "Xnnn gate [source [drain [sub]]]"		*/
@@ -427,7 +440,7 @@ spcdevHierVisit(hc, dev, scale)
 	    /* except that the "gate" node is treated as an identifier	*/
 	    /* only and is not output.					*/
 
-	    if (dev->dev_nterm > 1)
+	    if ((dev->dev_nterm > 1) && (dev->dev_class != DEV_MSUBCKT))
 		spcdevOutNode(hc->hc_hierName,
 			source->dterm_node->efnode_name->efnn_hier,
 			"subckt", esSpiceF);
@@ -452,7 +465,7 @@ spcdevHierVisit(hc, dev, scale)
 
 	    plist = efGetDeviceParams(EFDevTypes[dev->dev_type]);
 	    for (pptr = plist; pptr != NULL; pptr = pptr->parm_next)
-		if (pptr->parm_type == 's')
+		if (pptr->parm_type[0] == 's')
 		    break;
 
 	    if ((pptr == NULL) && subnode)
@@ -469,31 +482,102 @@ spcdevHierVisit(hc, dev, scale)
 	    sdM = getCurDevMult();
 	    while (plist != NULL)
 	    {
-		fprintf(esSpiceF, " %s=", plist->parm_name);
-		switch (plist->parm_type)
+		switch (plist->parm_type[0])
 		{
 		    case 'a':
-			if (esScale < 0)
-			    fprintf(esSpiceF, "%g", dev->dev_area * scale * scale);
-			else if (plist->parm_scale != 1.0)
-			    fprintf(esSpiceF, "%g", dev->dev_area * scale * scale
+			// Check for area of terminal node vs. device area
+			if (plist->parm_type[1] == '\0' || plist->parm_type[1] == '0')
+			{
+			    fprintf(esSpiceF, " %s=", plist->parm_name);
+			    parmval = dev->dev_area;
+			    if (esScale < 0)
+				fprintf(esSpiceF, "%g", parmval * scale * scale);
+			    else if (plist->parm_scale != 1.0)
+				fprintf(esSpiceF, "%g", parmval * scale * scale
 					* esScale * esScale * plist->parm_scale
 					* 1E-12);
-			else
-			    fprintf(esSpiceF, "%gp", dev->dev_area * scale * scale
+			    else
+				fprintf(esSpiceF, "%gp", parmval * scale * scale
 					* esScale * esScale);
+			}
+			else
+			{
+			    int pn;
+			    pn = plist->parm_type[1] - '0';
+			    if (pn >= dev->dev_nterm) pn = dev->dev_nterm - 1;
+
+			    dnode = GetHierNode(hc,
+				dev->dev_terms[pn].dterm_node->efnode_name->efnn_hier);
+
+			    // For parameter an followed by parameter pn,
+			    // process both at the same time
+
+			    if (plist->parm_next && plist->parm_next->parm_type[0] ==
+					'p' && plist->parm_next->parm_type[1] ==
+					plist->parm_type[1])
+			    {
+				spcnAP(dnode, esFetInfo[dev->dev_type].resClassSD,
+					scale, plist->parm_name, 
+					plist->parm_next->parm_name, sdM,
+					esSpiceF, w);
+				plist = plist->parm_next;
+			    }
+			    else
+			    {
+				spcnAP(dnode, esFetInfo[dev->dev_type].resClassSD,
+					scale, plist->parm_name, NULL, sdM,
+					esSpiceF, w);
+			    }
+			}
+	
 			break;
 		    case 'p':
-			if (esScale < 0)
-			    fprintf(esSpiceF, "%g", dev->dev_perim * scale);
-			else if (plist->parm_scale != 1.0)
-			    fprintf(esSpiceF, "%g", dev->dev_perim * scale
+			// Check for perimeter of terminal node vs. device perimeter
+			if (plist->parm_type[1] == '\0' || plist->parm_type[1] == '0')
+			{
+			    fprintf(esSpiceF, " %s=", plist->parm_name);
+			    parmval = dev->dev_perim;
+
+			    if (esScale < 0)
+				fprintf(esSpiceF, "%g", parmval * scale);
+			    else if (plist->parm_scale != 1.0)
+				fprintf(esSpiceF, "%g", parmval * scale
 					* esScale * plist->parm_scale * 1E-6);
+			    else
+				fprintf(esSpiceF, "%gu", parmval * scale * esScale);
+			}
 			else
-			    fprintf(esSpiceF, "%gu", dev->dev_perim * scale
-					* esScale);
+			{
+			    int pn;
+			    pn = plist->parm_type[1] - '0';
+			    if (pn >= dev->dev_nterm) pn = dev->dev_nterm - 1;
+
+			    dnode = GetHierNode(hc,
+				dev->dev_terms[pn].dterm_node->efnode_name->efnn_hier);
+
+			    // For parameter pn followed by parameter an,
+			    // process both at the same time
+
+			    if (plist->parm_next && plist->parm_next->parm_type[0] ==
+					'a' && plist->parm_next->parm_type[1] ==
+					plist->parm_type[1])
+			    {
+				spcnAP(dnode, esFetInfo[dev->dev_type].resClassSD,
+					scale, plist->parm_next->parm_name, 
+					plist->parm_name, sdM, esSpiceF, w);
+				plist = plist->parm_next;
+			    }
+			    else
+			    {
+				spcnAP(dnode, esFetInfo[dev->dev_type].resClassSD,
+					scale, NULL, plist->parm_name, sdM,
+					esSpiceF, w);
+			    }
+			}
+	
 			break;
 		    case 'l':
+			fprintf(esSpiceF, " %s=", plist->parm_name);
 			if (esScale < 0)
 			    fprintf(esSpiceF, "%g", l * scale);
 			else if (plist->parm_scale != 1.0)
@@ -503,6 +587,7 @@ spcdevHierVisit(hc, dev, scale)
 			    fprintf(esSpiceF, "%gu", l * scale * esScale);
 			break;
 		    case 'w':
+			fprintf(esSpiceF, " %s=", plist->parm_name);
 			if (esScale < 0)
 			    fprintf(esSpiceF, "%g", w * scale);
 			else if (plist->parm_scale != 1.0)
@@ -512,11 +597,13 @@ spcdevHierVisit(hc, dev, scale)
 			    fprintf(esSpiceF, "%gu", w * scale * esScale);
 			break;
 		    case 's':
+			fprintf(esSpiceF, " %s=", plist->parm_name);
 			subnodeFlat = spcdevSubstrate(hc->hc_hierName,
 				subnode->efnode_name->efnn_hier,
 				dev->dev_type, esSpiceF);
 			break;
 		    case 'x':
+			fprintf(esSpiceF, " %s=", plist->parm_name);
 			if (esScale < 0)
 			    fprintf(esSpiceF, "%g", dev->dev_rect.r_xbot * scale);
 			else if (plist->parm_scale != 1.0)
@@ -527,6 +614,7 @@ spcdevHierVisit(hc, dev, scale)
 					* esScale);
 			break;
 		    case 'y':
+			fprintf(esSpiceF, " %s=", plist->parm_name);
 			if (esScale < 0)
 			    fprintf(esSpiceF, "%g", dev->dev_rect.r_ybot * scale);
 			else if (plist->parm_scale != 1.0)
@@ -537,9 +625,11 @@ spcdevHierVisit(hc, dev, scale)
 					* esScale);
 			break;
 		    case 'r':
+			fprintf(esSpiceF, " %s=", plist->parm_name);
 			fprintf(esSpiceF, "%f", (double)(dev->dev_res));
 			break;
 		    case 'c':
+			fprintf(esSpiceF, " %s=", plist->parm_name);
 			fprintf(esSpiceF, "%ff", (double)(dev->dev_cap));
 			break;
 		}
@@ -693,10 +783,10 @@ spcdevHierVisit(hc, dev, scale)
 	    fprintf(esSpiceF, "\n+ ");
 	    dnode = GetHierNode(hc, drain->dterm_node->efnode_name->efnn_hier);
             spcnAP(dnode, esFetInfo[dev->dev_type].resClassSD, scale,
-			"d", sdM, esSpiceF, w);
+			"ad", "pd", sdM, esSpiceF, w);
 	    snode= GetHierNode(hc, source->dterm_node->efnode_name->efnn_hier);
 	    spcnAP(snode, esFetInfo[dev->dev_type].resClassSD, scale,
-			"s", sdM, esSpiceF, w);
+			"as", "ps", sdM, esSpiceF, w);
 	    if (subAP)
 	    {
 		fprintf(esSpiceF, " * ");
@@ -708,7 +798,7 @@ spcdevHierVisit(hc, dev, scale)
 		}
 		else if (subnodeFlat) 
 		    spcnAP(subnodeFlat, esFetInfo[dev->dev_type].resClassSub, scale, 
-	       			"sub", sdM, esSpiceF, -1);
+	       			"asub", "psub", sdM, esSpiceF, -1);
 		else
 		    fprintf(esSpiceF, "asub=0 psub=0");
 	    }
@@ -769,8 +859,8 @@ spcdevHierMergeVisit(hc, dev, scale)
 
     EFGetLengthAndWidth(dev, &l, &w);
 
-    fp = mkDevMerge(l * scale, w * scale, gnode, snode, dnode, subnode,
-		hc->hc_hierName, dev);
+    fp = mkDevMerge((float)((float)l * scale), (float)((float)w * scale),
+		gnode, snode, dnode, subnode, hc->hc_hierName, dev);
 		
     for (cfp = devMergeList; cfp != NULL; cfp = cfp->next)
     {
@@ -781,16 +871,18 @@ spcdevHierMergeVisit(hc, dev, scale)
 	    switch(dev->dev_class)
 	    {
 		case DEV_MOSFET:
+		case DEV_MSUBCKT:
 		case DEV_ASYMMETRIC:
 		case DEV_FET:
-		    m = esFMult[cfp->esFMIndex] + ((float)fp->w / (float)cfp->w);
+		    m = esFMult[cfp->esFMIndex] + (fp->w / cfp->w);
 		    break;
+		case DEV_RSUBCKT:
 		case DEV_RES:
 		    if (fp->dev->dev_type == esNoModelType)
 			m = esFMult[cfp->esFMIndex] + (fp->dev->dev_res
 				/ cfp->dev->dev_res);
 		    else
-			m = esFMult[cfp->esFMIndex] + ((float)fp->l / (float)cfp->l);
+			m = esFMult[cfp->esFMIndex] + (fp->l / cfp->l);
 		    break;
 		case DEV_CAP:
 		    if (fp->dev->dev_type == esNoModelType)
@@ -798,8 +890,7 @@ spcdevHierMergeVisit(hc, dev, scale)
 				/ cfp->dev->dev_cap);
 		    else
 			m = esFMult[cfp->esFMIndex] +
-				(((float)fp->l * (float)fp->w)
-				/ ((float)cfp->l * (float)cfp->w));
+				((fp->l * fp->w) / (cfp->l * cfp->w));
 		    break;
 	    }
 	    setDevMult(fp->esFMIndex, DEV_KILLED);
@@ -1019,7 +1110,7 @@ devMergeHierVisit(hc, dev, scale)
     /* Get length and width of the device */
     EFGetLengthAndWidth(dev, &l, &w);
 
-    fp = mkDevMerge((int)((float)l * scale), (int)((float)w * scale),
+    fp = mkDevMerge((float)((float)l * scale), (float)((float)w * scale),
 			gnode, snode, dnode, subnode, NULL, dev);
     hS = extHierSDAttr(source);
     hD = extHierSDAttr(drain);
@@ -1067,15 +1158,17 @@ mergeThem:
 	    {
 		case DEV_MOSFET:
 		case DEV_ASYMMETRIC:
+		case DEV_MSUBCKT:
 		case DEV_FET:
 		    m = esFMult[cfp->esFMIndex] + ((float)fp->w / (float)cfp->w);
 		    break;
+		case DEV_RSUBCKT:
 		case DEV_RES:
 		    if (fp->dev->dev_type == esNoModelType)
 		        m = esFMult[cfp->esFMIndex] + (fp->dev->dev_res
 				/ cfp->dev->dev_res);
 		    else
-		        m = esFMult[cfp->esFMIndex] + ((float)fp->l / (float)cfp->l);
+		        m = esFMult[cfp->esFMIndex] + (fp->l / cfp->l);
 		    break;
 		case DEV_CAP:
 		    if (fp->dev->dev_type == esNoModelType)
@@ -1083,8 +1176,7 @@ mergeThem:
 				/ cfp->dev->dev_cap);
 		    else
 		        m = esFMult[cfp->esFMIndex] +
-				(((float)fp->l  * (float)fp->w)
-				/ ((float)cfp->l * (float)cfp->w));
+				((fp->l  * fp->w) / (cfp->l * cfp->w));
 		    break;
 	    }
 	    setDevMult(fp->esFMIndex, DEV_KILLED); 
@@ -1322,6 +1414,7 @@ esHierVisit(hc, cdata)
     EFHierVisitResists(hcf, spcresistHierVisit, (ClientData)NULL);
 
     /* Output lumped capacitances */
+    sprintf( esSpiceCapFormat,  "C%%d %%s %%s %%.%dlffF\n", esCapAccuracy);
     EFHierVisitCaps(hcf, spccapHierVisit, (ClientData)NULL);
 
     if (def != topdef)
