@@ -54,7 +54,7 @@ static char sccsid[] = "@(#)ExtBasic.c	4.13 MAGIC (Berkeley) 12/5/85";
 
 #ifdef MAGIC_WRAPPER
 char *extDevTable[] = {"fet", "mosfet", "asymmetric", "bjt", "devres",
-	"devcap", "diode", "subckt", "rsubckt", NULL};
+	"devcap", "diode", "subckt", "rsubckt", "msubckt", NULL};
 #endif
 
 /* --------------------- Data local to this file ---------------------- */
@@ -330,7 +330,7 @@ extSetResist(reg)
 	extResistArea[n] = extResistPerim[n] = 0;
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -800,7 +800,7 @@ extComputeEffectiveLW(rlengthptr, rwidthptr, numregions, chop)
 
         for (i = 0; i < numregions; i++)
 	{
-	    if (i == p) continue;
+	    if ((i == p) && (i > 1)) continue;
 	    for (lb2 = extSpecialBounds[i]; lb2 != NULL; lb2 = lb2->b_next)
 	    {
 		if (lb2->dir == oppdir)
@@ -920,7 +920,7 @@ extComputeEffectiveLW(rlengthptr, rwidthptr, numregions, chop)
 	cornerw = 0;
         for (i = 0; i < numregions; i++)
 	{
-	    if (i == p) continue;
+	    if ((i == p) && (i > 1)) continue;
 	    for (lb2 = extSpecialBounds[i]; lb2 != NULL; lb2 = lb2->b_next)
 	    {
 		if (lb2->dir == oppdir)
@@ -995,7 +995,14 @@ extComputeEffectiveLW(rlengthptr, rwidthptr, numregions, chop)
     if ((length > 0) && (width > 0))
     {
 	*rlengthptr = length;
-	*rwidthptr = (width >> 1);
+
+	// If numregions == 1 then everything was put in one record,
+	// and we have double-counted the width.
+
+	if (numregions == 1)
+	    *rwidthptr = (width >> 2);
+	else
+	    *rwidthptr = (width >> 1);
 
 	/* fprintf(stderr, "total L = %d, W = %d\n", length, width); */
 	/* fflush(stderr); */
@@ -1127,9 +1134,9 @@ extSeparateBounds(nterm)
  *
  * Scan through the TransRegion in the supplied list, and collect a mask of
  * all transistor types used in the layout.  Then for each transistor type,
- * find if it belongs to a "subcircuit" or "rsubcircuit" definition.  If it
- * does, output a record containing the list of parameter names used by that
- * subcircuit.
+ * find if it belongs to a "subcircuit" (including "rsubcircuit" and
+ * "msubcircuit") definition.  If it does, output a record containing the
+ * list of parameter names used by that subcircuit.
  *
  * Results:
  *	None.
@@ -1165,11 +1172,26 @@ extOutputParameters(def, transList, outFile)
 		fprintf(outFile, "parameters %s", ExtCurStyle->exts_transName[t]);
 		for (; plist != NULL; plist = plist->pl_next)
 		{
-		    if (plist->pl_scale != 1.0)
-			fprintf(outFile, " %c=%s*%g", plist->pl_param,
+		    if (plist->pl_param[1] != '\0')
+		    {
+			if (plist->pl_scale != 1.0)
+			    fprintf(outFile, " %c%c=%s*%g", 
+					plist->pl_param[0], plist->pl_param[1],
 					plist->pl_name, plist->pl_scale);
+			else
+			    fprintf(outFile, " %c%c=%s", plist->pl_param[0],
+					plist->pl_param[1], plist->pl_name);
+		    }
 		    else
-			fprintf(outFile, " %c=%s", plist->pl_param, plist->pl_name);
+		    {
+			if (plist->pl_scale != 1.0)
+			    fprintf(outFile, " %c=%s*%g", 
+					plist->pl_param[0],
+					plist->pl_name, plist->pl_scale);
+			else
+			    fprintf(outFile, " %c=%s", plist->pl_param[0],
+					plist->pl_name);
+		    }
 		}
 		fprintf(outFile, "\n");
 	    }
@@ -1177,6 +1199,53 @@ extOutputParameters(def, transList, outFile)
     }
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * extGetNativeResistClass() --
+ *
+ * For the purpose of generating a node area and perimeter value to output
+ * to a subcircuit call as a passed parameter.  The value output is assumed
+ * to refer only to the part of the whole eletrical node that is the
+ * actual device node, not to include connected metal, contacts, etc.
+ * Since area and perimeter information about a node is separated into
+ * resist classes, we need to figure out which resist class belongs to
+ * the device terminal type.
+ *
+ * "type" is the type identifier for the device (e.g., gate).  "term" is
+ * the index of the terminal for the device.  Devices with symmetrical
+ * terminals (e.g., MOSFETs), may have fewer type masks than terminals.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+extGetNativeResistClass(type, term)
+    TileType type;
+    int term;
+{
+    TileTypeBitMask *tmask, *rmask;
+    int i, n;
+
+    tmask = NULL;
+    for (i = 0;; i++)
+    {
+	rmask = &ExtCurStyle->exts_transSDTypes[type][i];
+	if (TTMaskHasType(rmask, TT_SPACE)) break;
+	tmask = rmask;
+	if (i == term) break;
+    }
+    if (tmask == NULL) return -1;	/* Error */
+
+    for (n = 0; n < ExtCurStyle->exts_numResistClasses; n++)
+    {
+	rmask = &ExtCurStyle->exts_typesByResistClass[n];
+	if (TTMaskIntersect(rmask, tmask))
+	    return n;
+    }
+    return -1; 		/* Error */
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1220,7 +1289,7 @@ extOutputTrans(def, transList, outFile)
     FindRegion arg;
     LabelList *ll;
     TileType t;
-    int nsd, length, width, n, i, ntiles, corners;
+    int nsd, length, width, n, i, ntiles, corners, tn, rc;
     double dres, dcap;
     char mesg[256];
     bool isAnnular, hasModel;
@@ -1361,6 +1430,7 @@ extOutputTrans(def, transList, outFile)
 	    case DEV_MOSFET:
 	    case DEV_BJT:
 	    case DEV_SUBCKT:
+	    case DEV_MSUBCKT:
 	    case DEV_ASYMMETRIC:
 		length = extTransRec.tr_gatelen / 2;	/* (default) */
 		width = 0;
@@ -1446,31 +1516,36 @@ extOutputTrans(def, transList, outFile)
 
 		}
 
-		if (ExtCurStyle->exts_deviceClass[t] == DEV_SUBCKT)
+		if (ExtCurStyle->exts_deviceClass[t] == DEV_SUBCKT ||
+			ExtCurStyle->exts_deviceClass[t] == DEV_MSUBCKT)
 		{
 		    for (chkParam = ExtCurStyle->exts_deviceParams[t]; chkParam
 				!= NULL; chkParam = chkParam->pl_next)
 		    {
-			switch(tolower(chkParam->pl_param))
+			switch(tolower(chkParam->pl_param[0]))
 			{
 			    case 'a':
-				fprintf(outFile, " %c=%d", chkParam->pl_param,
+				if (chkParam->pl_param[1] == '\0' ||
+					chkParam->pl_param[1] == '0')
+				    fprintf(outFile, " %c=%d", chkParam->pl_param[0],
 						reg->treg_area);
 				break;
 			    case 'p':
-				fprintf(outFile, " %c=%d", chkParam->pl_param,
+				if (chkParam->pl_param[1] == '\0' ||
+					chkParam->pl_param[1] == '0')
+				    fprintf(outFile, " %c=%d", chkParam->pl_param[0],
 						extTransRec.tr_perim);
 				break;
 			    case 'l':
-				fprintf(outFile, " %c=%d", chkParam->pl_param,
+				fprintf(outFile, " %c=%d", chkParam->pl_param[0],
 						length);
 				break;
 			    case 'w':
-				fprintf(outFile, " %c=%d", chkParam->pl_param,
+				fprintf(outFile, " %c=%d", chkParam->pl_param[0],
 						width);
 				break;
 			    case 'c':
-				fprintf(outFile, " %c=%g", chkParam->pl_param,
+				fprintf(outFile, " %c=%g", chkParam->pl_param[0],
 					(ExtCurStyle->exts_transGateCap[t]
 					* reg->treg_area) +
 					(ExtCurStyle->exts_transSDCap[t]
@@ -1482,7 +1557,7 @@ extOutputTrans(def, transList, outFile)
 				/* Do nothing;  these values are standard output */
 				break;
 			    default:
-				fprintf(outFile, " %c=", chkParam->pl_param);
+				fprintf(outFile, " %c=", chkParam->pl_param[0]);
 				break;
 			}
 		    }
@@ -1616,30 +1691,34 @@ extOutputTrans(def, transList, outFile)
 		    for (chkParam = ExtCurStyle->exts_deviceParams[t]; chkParam
 				!= NULL; chkParam = chkParam->pl_next)
 		    {
-			switch(tolower(chkParam->pl_param))
+			switch(tolower(chkParam->pl_param[0]))
 			{
 			    case 'a':
-				fprintf(outFile, " %c=%d", chkParam->pl_param,
-					reg->treg_area);
+				if (chkParam->pl_param[1] == '\0' ||
+					chkParam->pl_param[1] == '0')
+				    fprintf(outFile, " %c=%d", chkParam->pl_param[0],
+						reg->treg_area);
 				break;
 			    case 'p':
-				fprintf(outFile, " %c=%d", chkParam->pl_param,
-					extTransRec.tr_perim);
+				if (chkParam->pl_param[1] == '\0' ||
+					chkParam->pl_param[1] == '0')
+				    fprintf(outFile, " %c=%d", chkParam->pl_param[0],
+						extTransRec.tr_perim);
 				break;
 			    case 'l':
-				fprintf(outFile, " %c=%d", chkParam->pl_param,
+				fprintf(outFile, " %c=%d", chkParam->pl_param[0],
 					length);
 				break;
 			    case 'w':
-				fprintf(outFile, " %c=%d", chkParam->pl_param,
+				fprintf(outFile, " %c=%d", chkParam->pl_param[0],
 					width);
 				break;
 			    case 'r':
-				fprintf(outFile, " %c=%g", chkParam->pl_param,
+				fprintf(outFile, " %c=%g", chkParam->pl_param[0],
 					dres / 1000.0);
 				break;
 			    case 'c':
-				fprintf(outFile, " %c=%g", chkParam->pl_param,
+				fprintf(outFile, " %c=%g", chkParam->pl_param[0],
 					(ExtCurStyle->exts_transGateCap[t]
 					* reg->treg_area) +
 					(ExtCurStyle->exts_transSDCap[t]
@@ -1651,7 +1730,7 @@ extOutputTrans(def, transList, outFile)
 				/* Do nothing;  these values are standard output */
 				break;
 			    default:
-				fprintf(outFile, " %c=", chkParam->pl_param);
+				fprintf(outFile, " %c=", chkParam->pl_param[0]);
 				break;
 			}
 		    }
