@@ -402,8 +402,20 @@ extHardGenerateLabel(scx, reg, arg)
     char gen[100];
     int len;
     Rect r;
+    Point p;
+
+    // Modification 9/9/2014 by Tim:
+    // Convert the treg_ll value up to top-level coordinates.
+    // Otherwise you end up with a node that is apparently in
+    // "canonical coordinates", but if you try to find the
+    // location of the node using the name, you'll end up in
+    // a random place.  It also allows the low-probability
+    // but possible conflict between this node and another with
+    // the same name in the parent cell.
+
+    GeoTransPoint(&scx->scx_trans, &reg->treg_ll, &r.r_ll);
+    extMakeNodeNumPrint(gen, reg->treg_pnum, r.r_ll);
 
-    extMakeNodeNumPrint(gen, reg->treg_pnum, reg->treg_ll);
     prefixlen = tpath->tp_next - tpath->tp_first;
     len = strlen(gen) + prefixlen;
     newlab = (Label *) mallocMagic((unsigned) (sizeof (Label) + len - 3));
@@ -417,6 +429,9 @@ extHardGenerateLabel(scx, reg, arg)
     /* Don't care, really, which orientation the label has */
     newlab->lab_just = GEO_NORTH;
 
+    /* Mark this as a generated label;  may or may not be useful */
+    newlab->lab_flags = LABEL_GENERATE;
+
     /* Construct the text of the new label */
     dstp = newlab->lab_text;
     if (prefixlen)
