@@ -170,7 +170,7 @@ static keydesc keyTable[] = {
  * Table used for parsing the "device" keyword types
  *
  * (Note: "10" for max types in subcircuit is arbitrary---the parser
- * ignores max types for DEV_SUBCKT).
+ * ignores max types for DEV_SUBCKT and DEV_MSUBCKT).
  */
 
 /* types are enumerated in extract.h */
@@ -197,6 +197,9 @@ static keydesc devTable[] = {
     "rsubcircuit",	DEV_RSUBCKT,		4,	7,
 "name dev-types terminal-types [sub-types|None sub-node] [options]",
 
+    "msubcircuit",	DEV_MSUBCKT,		4,	11,
+"name dev-types [N] [term1-types ... termN-types [sub-types|None sub-node]] [options]",
+
     0
 };
 
@@ -1872,13 +1875,14 @@ ExtTechLine(sectionName, argc, argv)
 	    if ((argc - 1) < dv->k_minargs)
 		goto usage;
 
-	    /* No limit on arguments in DEV_SUBCKT! */
+	    /* No limit on arguments in DEV_SUBCKT and DEV_MSUBCKT! */
 	    /* And. . . check DEV_RSUBCKT later, after parsing parameter names */
 	    class = dv->k_key;
 	    switch (class)
 	    {
 		case DEV_SUBCKT:
 		case DEV_RSUBCKT:
+		case DEV_MSUBCKT:
 		    break;
 		default:
 		    if ((argc - 1) > dv->k_maxargs)
@@ -1985,6 +1989,7 @@ ExtTechLine(sectionName, argc, argv)
 		    break;
 
 		case DEV_SUBCKT:
+		case DEV_MSUBCKT:
 		    /* Check final arguments for "x=y" statements showing what	*/
 		    /* parameter names the subcircuit uses.			*/
 
@@ -1995,7 +2000,15 @@ ExtTechLine(sectionName, argc, argv)
 			paramName++;
 			newParam = (ParamList *)mallocMagic(sizeof(ParamList));
 			newParam->pl_count = 0;
-			newParam->pl_param = *argv[argc - 1];
+			newParam->pl_param[0] = *argv[argc - 1];
+			newParam->pl_param[1] = '\0';
+
+			if (paramName - argv[argc - 1] == 3)
+			    newParam->pl_param[1] = *(argv[argc - 1] + 1);
+
+			else if (paramName - argv[argc - 1] > 3)
+			    TechError("Parameter name %s can be no more than"
+				"two characters.\n", argv[argc - 1]);
 
 			// Parameter syntax "<type>=<name>*<scale>" indicates
 			// that the subcircuit has internal scaling, and the
@@ -2035,6 +2048,12 @@ ExtTechLine(sectionName, argc, argv)
 
 		    if (nterm == 0) i++;
 
+		    // Type MSUBCKT:  Source and drain are symmetric.  The
+		    // number of unique terminals in the definition is 1,
+		    // but nterm needs to be set to 2 for proper extraction.
+
+		    if ((dv->k_key == DEV_MSUBCKT) && (nterm == 1)) nterm = 2;
+
 		    if (argc > i)
 			DBTechNoisyNameMask(argv[i], &subsTypes); /* substrate */
 		    if (argc > (i + 1)) subsName = argv[i + 1];
@@ -2051,9 +2070,17 @@ ExtTechLine(sectionName, argc, argv)
 			paramName++;
 			newParam = (ParamList *)mallocMagic(sizeof(ParamList));
 			newParam->pl_count = 0;
-			newParam->pl_param = *argv[argc - 1];
+			newParam->pl_param[0] = *argv[argc - 1];
+			newParam->pl_param[1] = '\0';
 
-			// See comments for DEV_SUBCKT above.
+			if (paramName - argv[argc - 1] == 3)
+			    newParam->pl_param[1] = *(argv[argc - 1] + 1);
+
+			else if (paramName - argv[argc - 1] > 3)
+			    TechError("Parameter name %s can be no more than"
+				"two characters.\n", argv[argc - 1]);
+
+			// See comments for DEV_SUBCKT/DEV_MSUBCKT above.
 
 			if ((mult = strchr(paramName, '*')) != NULL)
 			{
