@@ -67,6 +67,9 @@ extern bool GrTOGLInit();
 extern bool GrTOGLEventPending(), GrTOGLCreate(), grtoglGetCursorPos();
 extern int  GrTOGLWindowId();
 extern char *GrTkWindowName();
+
+extern void toglSetProjection();
+
 
 /*---------------------------------------------------------
  * grtoglSetWMandC:
@@ -376,7 +379,7 @@ static GLXPixmap glpmap;
  *---------------------------------------------------------
  */
 
-int
+void
 toglSetProjection(llx, lly, width, height)
     int llx, lly, width, height;
 {
@@ -519,7 +522,7 @@ keys_and_buttons:
 
 		keymod = (LockMask | ControlMask | ShiftMask)
 				& KeyPressedEvent->state;
-#ifdef macosx
+#ifdef __APPLE__
 		if (KeyPressedEvent->state & (Mod1Mask | Mod2Mask |
 				Mod3Mask | Mod4Mask | Mod5Mask))
 		    keymod |= Mod1Mask;
@@ -804,7 +807,7 @@ keys_and_buttons:
 			mw->w_flags &= ~WIND_OBSCURED;
 			if (mw->w_backingStore == (ClientData)NULL)
 			{
-			    grtkCreateBackingStore(mw);
+			    (*GrCreateBackingStorePtr)(mw);
 			    if (mw->w_backingStore != (ClientData)NULL)
 			    {
 				WindAreaChanged(mw, &mw->w_allArea);
@@ -946,11 +949,19 @@ oglSetDisplay (dispType, outFileName, mouseFileName)
     grSetCharSizePtr = grtoglSetCharSize;
     grFillPolygonPtr = grtoglFillPolygon;
 
+#ifdef X11_BACKING_STORE
     GrFreeBackingStorePtr = grtkFreeBackingStore;
     GrCreateBackingStorePtr = grtkCreateBackingStore;
     GrGetBackingStorePtr = grtkGetBackingStore;
     GrPutBackingStorePtr = grtkPutBackingStore;
     GrScrollBackingStorePtr = grtkScrollBackingStore;
+#else
+    GrFreeBackingStorePtr = grtoglFreeBackingStore;
+    GrCreateBackingStorePtr = grtoglCreateBackingStore;
+    GrGetBackingStorePtr = grtoglGetBackingStore;
+    GrPutBackingStorePtr = grtoglPutBackingStore;
+    GrScrollBackingStorePtr = grtoglScrollBackingStore;
+#endif
     
     if (execFailed) {
 	TxError("Execution failed!\n");
