@@ -191,7 +191,7 @@ proc magic::setgrid {gridsize} {
    set tech1 [lindex $techlambda 1]
    set tech0 [lindex $techlambda 0]
    set tscale [expr {$tech1 / $tech0}]
-   set lambdaout [expr {[magic::cif lambda output] * $tscale}]
+   set lambdaout [expr {[magic::cif scale output] * $tscale}]
    set gridlambda [expr {$gridsize/$lambdaout}]
    magic::grid ${gridlambda}l
    magic::snap on
@@ -390,10 +390,10 @@ proc magic::techmanager {{option "update"}} {
    set techstuff [magic::tech version]
    .techmgr.title.tvers configure -text "(version [lindex $techstuff 0])"
    .techmgr.subtitle configure -text [lindex $techstuff 1]
-   set lotext [format "%g" [expr {[magic::cif lambda output] * $tscale}]]
+   set lotext [format "%g" [expr {[magic::cif scale output] * $tscale}]]
    .techmgr.lambda0.lval configure -text $lotext
    .techmgr.cif0.lstyle configure -text [magic::cif list istyle]
-   set litext [format "%g" [expr {[magic::cif lambda input] * $tscale}]]
+   set litext [format "%g" [expr {[magic::cif scale input] * $tscale}]]
    .techmgr.cif0.llambda configure -text $litext
    .techmgr.cif1.lstyle configure -text [magic::cif list ostyle]
    .techmgr.cif1.llambda configure -text $lotext
@@ -557,7 +557,7 @@ proc magic::cursorview {win} {
    set lambda [${win} tech lambda]
    if {[llength $lambda] != 2} {return}
    set lr [expr {(0.0 + [lindex $lambda 0]) / [lindex $lambda 1]}]
-   set olst [${win} cursor lambda]
+   set olst [${win} cursor microns]
    set olstx [lindex $olst 0]
    set olsty [lindex $olst 1]
 
@@ -573,7 +573,7 @@ proc magic::cursorview {win} {
       if {[expr {$dy == int($dy)}]} {set dy [expr {int($dy)}]}
       if {$dx >= 0} {set dx "+$dx"}
       if {$dy >= 0} {set dy "+$dy"}
-      set titletext [format "(%g %g) %g %g lambda" $olstx $olsty $dx $dy]
+      set titletext [format "(%g %g) %g %g microns" $olstx $olsty $dx $dy]
       ${framename}.titlebar.pos configure -text $titletext
    } else {
       set titletext [format "(%g %g) lambda" $olstx $olsty]
